%{
#include <math.h>
#include <string.h>
#include "eva.h"
#ifdef _cgi_
# include "eva-cgi.tab.h"
#else
# include "eva.tab.h"
#endif
%}

%option noyywrap

INT [0-9]+

FLOAT (([0-9]+(\.[0-9]*)|[0-9]+)e[+-]?[0-9]+|[0-9]+\.[0-9]+)

%%

{FLOAT} {
  yylval.val = atof(yytext);
  return NUM;
}

{INT} {
  yylval.val = atoi(yytext);
  return NUM;
}

quit|QUIT return QUIT;

Eva|eva|EVA return EVA;

Adam|adam|ADAM return ADAM;

%28 return '(';

%29 return ')';

[a-zA-Z_][a-zA-Z0-9_]* {
  if (!(yylval.sym = get_symbol(yytext)))
    yylval.sym = define_symbol(yytext, VAR);
  return yylval.sym->type;
}

\  {};

\n return *yytext;

. return *yytext;

%%

int parse_string(const char *s) {
  int result;

  char *string = malloc(strlen(s) + 2);
  strcpy(string, s);
  strcat(string, "\n");

  YY_BUFFER_STATE yy_buffer = yy_scan_string(string);
  result = yyparse();
  yy_delete_buffer(yy_buffer);
  return result;
}

void yyerror(const char *s) {
  printf ("%s\n", s);
}

double exp10(double val) {
  return pow(10.0, val);
}

extern int yyparse(void);
char *input_string = 0;
int input_string_len;
extern int yydebug;

void set_input_string(const char *s) {
  input_string_len = strlen(s);
  input_string = malloc(input_string_len + 1);
  strcpy(input_string, s);
}

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

struct init_struct {
  char *fname;
  double (*fnct)(double);
};

struct init_struct single_argument_functions[] = {
  { "sin",   sin    },
  { "asin",  asin   },
  { "cos",   cos    },
  { "acos",  acos   },
  { "tan",   tan    },
  { "atan",  atan   },
  { "sinh",  sinh   },
  { "cosh",  cosh   },
  { "tanh",  tanh   },
  { "asinh", sinh   },
  { "acosh", cosh   },
  { "atanh", atanh  },
  { "sqrt",  sqrt   },
  { "cbrt",  cbrt   },
  { "exp",   exp    },
  { "exp2",  exp2   },
  { "exp10", exp10  },
  { "ln",    log    },
  { "ln2",   log2   },
  { "ln10",  log10  },
  { "erf",   erf    },
  { "erfc",  erfc   },
  { "gamma", tgamma },
  { 0, 0 }
};

struct sym_def_struct *symbol_table = 0;

struct sym_def_struct *define_symbol(const char *sym_name, int sym_type) {
  struct sym_def_struct *ptr;
  ptr = (struct sym_def_struct *) malloc(sizeof(struct sym_def_struct));
  ptr->name = (char *) malloc(strlen(sym_name) + 1);
  strcpy(ptr->name, sym_name);
  ptr->type = sym_type;
  ptr->value.var = 0;
  ptr->next = symbol_table;
  symbol_table = ptr;
  return ptr;
}

struct sym_def_struct *get_symbol(const char *sym_name) {
  struct sym_def_struct *ptr;
  for (ptr = symbol_table; ptr; ptr = ptr->next)
    if (strcmp(ptr->name,sym_name) == 0)
      return ptr;
  return 0;
}

void print_val(double val) {
  if ((val > 0.001) && (val < 100000)) printf("%.10g\n", val);
  else printf("%.10e\n", val);
}

void init(void) {
  int i;
  struct sym_def_struct *ptr;
  for (i = 0; single_argument_functions[i].fname != 0; i++) {
    ptr = define_symbol(single_argument_functions[i].fname, FNCT);
    ptr->value.func = single_argument_functions[i].fnct;
  }
  ptr = define_symbol("pi", VAR);
  ptr->value.var = 4.0 * atan(1.0);
  ptr = define_symbol("e", VAR);
  ptr->value.var = exp(1.0);
  ptr = define_symbol("c", VAR);
  ptr->value.var = 3e8;
}

void quit(void) {
  exit(0);
}
